#define F_CPU 11059200UL
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <uart.c>

#define ddT  DDRB
#define TPort  PORTB
#define  Tp PB2
#define TPin PINB

#define tLow() TPort&=~(1<<Tp)
#define tHigh() TPort|=(1<<Tp)
#define tIn() ddT|=(1<<Tp)
#define tOut() ddT&=~(1<<Tp)

#define CONVERTTEMP 0x44
#define RSCRATCHPAD 0xbe
#define WSCRATCHPAD 0x4e
#define CPYSCRATCHPAD 0x48
#define RECEEPROM 0xb8
#define RPWRSUPPLY 0xb4
#define SEARCHROM 0xf0
#define READROM 0x33
#define MATCHROM 0x55
#define SKIPROM 0xcc
#define ALARMSEARCH 0xec

uint8_t rPulse(void){
	uint8_t i;
	tLow();
	tOut();
	_delay_us(480);
	tIn(); 
	_delay_us(60);
	i=(TPin & (1<<Tp));
	_delay_us(420);
//Return the value read from the presence pulse (0=OK, 1=WRONG)
return i;
}


void write_bit(uint8_t bit){
//Pull line low for 1uS
tLow();
tOut();
_delay_us(1);
//If we want to write 1, release the line (if not will keep low)
if(bit) tIn();
//Wait for 60uS and release the line
_delay_us(60);
tIn();
}

uint8_t read_bit(void){
uint8_t bit=0;
//Pull line low for 1uS
tLow();
tOut();
_delay_us(1);
//Release line and wait for 14uS
tIn();
_delay_us(14);
//Read line value
if(TPin&(1<<Tp)) bit=1;
//Wait for 45uS to end and return read value
_delay_us(45);
return bit;
}

void write_byte(uint8_t val){
uint8_t i=8;
while(i--){
//Write actual bit and shift one position right to make the next bit ready
write_bit(val&1);
val>>=1;
}
}	

uint8_t read_byte(void){
uint8_t i=8, n=0;
while(i--){
//Shift one position right and store read value
n>>=1;
n|=(read_bit()<<7);
}
return n;
}

void  readTemp(void){
uint8_t temperature[2];

//Reset, skip ROM and start temperature conversion
rPulse();
uart_putc('c');
write_byte(SKIPROM);
write_byte(CONVERTTEMP);
uart_putc('d');
//Wait until conversion is complete
while(!read_bit());
//Reset, skip ROM and send command to read Scratchpad
rPulse();
uart_putc('e');
write_byte(SKIPROM);
write_byte(RSCRATCHPAD);
uart_putc('f');
//Read Scratchpad (only 2 first bytes)
temperature[0]=read_byte();
temperature[1]=read_byte();
rPulse();
uart_putc(temperature[0]);
uart_putc(temperature[1]);
}

int main(void){
	DDRB=0x06;
	PORTB=0x00;
	usartInit();
	tOut();
	rPulse();
write_byte(READROM);
uint8_t byte;
byte=read_byte();
uart_putc(byte);
byte=read_byte();
uart_putc(byte);
byte=read_byte();
uart_putc(byte);
byte=read_byte();
uart_putc(byte);
byte=read_byte();
uart_putc(byte);
byte=read_byte();
uart_putc(byte);
byte=read_byte();
uart_putc(byte);
byte=read_byte();
uart_putc(byte);


//ds1820init();
while(0x01==rPulse())
	;
		while(1){
			uart_putc('b');
		readTemp();
			_delay_ms(400);
			}
}  

